#include <SDL/SDL.h>
#include <xmms/plugin.h>  
#include <stdlib.h>
#include <glib.h>
#include "tinyptc.h"
#include <math.h>

#include "goom.h"
#include "graphic.h"
#include "filters.h"
#include "goomtools.h"

unsigned int POINT_WIDTH ;
unsigned int POINT_HEIGHT ;
#define STOP_SPEED 128

static void jeko_init(void);
static void jeko_cleanup(void);
static void jeko_playback_start(void);
static void jeko_playback_stop(void);
static void jeko_render_pcm(gint16 data[2][512]);

void jeko_update_config () ;

VisPlugin jeko_vp =
{
  NULL,
  NULL,
  0,
  "What a GOOM! "VERSION,
  2,
  0,
  jeko_init, /* init */
  jeko_cleanup, /* cleanup */
  about, /* about */
  plug_configure, /* configure */
  NULL, /* disable_plugin */
  jeko_playback_start, /* playback_start */
  jeko_playback_stop, /* playback_stop */
  jeko_render_pcm, /* render_pcm */
  /* jeko_render_freq*/ NULL  /* render_freq */
};

/*-----------------------------------------------------
SHARED DATA
-------------------------------------------------------*/
	Uint *pixel ;
	Uint *back ;
	Uint *p1,*p2,*tmp;
	Uint x,y;
	Uint cycle;
	Uint debut, fin;
	char fast ;


VisPlugin *get_vplugin_info(void) 
{    
  return &jeko_vp;
}   

/*======================*/
/*===============================*/
void jeko_init(void)
{	
	printf ("--> INIT\n") ;
	plug_load_prefs () ;
	jeko_update_config () ;

	/* ouverture */
	if (!ptc_open("What a GOOM!",WIDTH,HEIGHT)) exit (1);
	
	printf ("Buffers size : %i\n", SIZE) ;
	
	pixel = (Uint *) malloc (SIZE * sizeof(Uint)) ;
	back = (Uint *) malloc (SIZE * sizeof(Uint)) ;
	RAND_INIT (pixel) ;
}

static void jeko_cleanup(void)
{
	printf ("--> CLEANUP\n") ;
	/* fermeture */
	ptc_close();
	free (pixel) ;
	free (back) ;
	RAND_CLOSE();
}

static void jeko_playback_start(void)
{
	printf ("--> START\n") ;
	cycle = 0;

	/* remplissage de la grille */
	for (y=0;y<HEIGHT;y++) for (x=0;x<WIDTH;x++) {
		if ( (x/8)%2 == (y/8)%2 )
			setPixelRGB(pixel,x,y,BLACK);
		else
			setPixelRGB(pixel,x,y,VIOLET);
	}
	
	/* boucle */
	p1=pixel;
	p2=back;

	RAND_INIT ((unsigned int)cycle * (unsigned int)p1) ;
}

static void jeko_playback_stop(void)
{
	printf ("--> STOP\n") ;
}

static void jeko_render_pcm(gint16 data[2][512])
{
	static int lockvar = 0 ; // pour empecher de nouveaux changements

	static int goomvar = 0 ; // boucle des gooms
	static int totalgoom = 0 ; // nombre de gooms par seconds
	static int totalgrosgoom = 0 ; // nombre de grosgooms par seconds
	static char goomlimit = 2 ; // sensibilit du goom
	static char grosgoomlimit = 2 ; // sensibilite ajoute pour gros goom
	
	static int agoom = 0 ; // un goom a eu lieu..
	
	static int loopvar = 0 ; // mouvement des points
	int incvar ; // volume du son
	
	static int speedvar = 0 ; // vitesse des particules
	int newspeedvar ;
	
	int accelvar ; // acceleration des particules
	int i ;
	float largfactor ; // elargissement de l'intervalle d'volution des points
	
	static ZoomFilterData zfd = {128,8,16,1,1,0, WAVE_MODE, 0,0, 0} ;
	ZoomFilterData *pzfd ;
	
	static int sinrate = 0 ;
	static int sintimeleft = 0 ;
	
	static unsigned int PREVIOUS_SIZE ;

	/* test if the config has changed, update it if so */
	PREVIOUS_SIZE = SIZE ;
	jeko_update_config () ;
	if (PREVIOUS_SIZE != SIZE) {
		PREVIOUS_SIZE = SIZE ;
		jeko_cleanup () ;
		SDL_Delay (1000) ;
		jeko_init () ;
		jeko_playback_start () ;
	}

	incvar = 0 ;
	for (i=0;i<512;i++)
	{
		if (incvar<data[0][i]) incvar = data[0][i] ;
	}

	accelvar = incvar / 5000 ;
	if (speedvar>5)
	{
		accelvar-- ;
		if (speedvar>20) accelvar -- ;
		if (speedvar>40) speedvar = 40 ;
	}
	accelvar -- ;
	speedvar += accelvar ;
	
	if (speedvar<0) speedvar=0;

	largfactor = ((float)speedvar / 40.0f + (float)incvar / 50000.0f) / 1.5f ;
	if (largfactor>1.0f) largfactor = 1.0f ;

	for (i = 1 ; i*15 <= speedvar + 15; i ++) {
		loopvar += speedvar + 1 ;
		
		pointFilter(p1, YELLOW,
			((POINT_WIDTH-6.0f)*largfactor+5.0f),
			((POINT_HEIGHT-6.0f)*largfactor+5.0f),
			i*76.0f, 64.0f, loopvar+i*2032);
		pointFilter(p1, ORANGE,
			((POINT_WIDTH/2)*largfactor)/i+10.0f*i,
			((POINT_HEIGHT/2)*largfactor)/i+10.0f*i,
			48.0f, i*40.0f, loopvar/i);
		pointFilter(p1, VIOLET,
			((POINT_HEIGHT/3+5.0f)*largfactor)/i+10.0f*i,
			((POINT_HEIGHT/3+5.0f)*largfactor)/i+10.0f*i,
			i+61.0f, 67.0f, loopvar/i);
		pointFilter(p1, BLACK,
			((POINT_HEIGHT/3)*largfactor+20.0f),
			((POINT_HEIGHT/3)*largfactor+20.0f),
			29.0f, i*33.0f, loopvar/i);
		pointFilter(p1, WHITE,
			(POINT_HEIGHT*largfactor+10.0f*i)/i,
			(POINT_HEIGHT*largfactor+10.0f*i)/i,
			33.0f, 37.0f, loopvar+i*500);
	}

	// par dfaut pas de changement de zoom
	pzfd = NULL ;
	
	// diminuer de 1 le temps de lockage
	lockvar -- ;
	if (lockvar==-1) lockvar = 0 ;

	// temps du goom
	agoom -- ;
	if (agoom<0) agoom = 0 ;
	if ((accelvar>goomlimit) || (accelvar<-goomlimit))
	{
		totalgoom ++ ;
		agoom = 5 ;
		
		// changement eventuel de mode
		// if (cycle%1==0)
			switch ((int)(10.0f*rand()/RAND_MAX))
			{
			case 0:
			case 1:
			case 2:
				zfd.mode=WAVE_MODE;
				zfd.vitesse=STOP_SPEED-1;
				zfd.reverse=0;
				break;
			case 3:
			case 4:
				zfd.mode=CRYSTAL_BALL_MODE;
				break;
			case 5:
				zfd.mode=AMULETTE_MODE;
				break;
			case 6:
			case 7:
				zfd.mode=SCRUNCH_MODE;
				break;
			default:
				zfd.mode=NORMAL_MODE;
			}
	}

	// gros goom : on active le filtre sin
	if (((accelvar>goomlimit+grosgoomlimit)
		|| (accelvar<-goomlimit-grosgoomlimit))
		&& (sinrate<40) )
	{
		if (totalgrosgoom<3) {
			sinrate += accelvar * 10 ;
			sintimeleft = 20 ;
		}
		totalgrosgoom ++ ;
	}
	
	// filtre sin activ ? : on diminue son intensit...
	if ((sinrate>0) && (cycle%2==0))
	{
		sinrate -- ;
		if (sinrate==0) {
			zfd.vitesse += 10 ;
			if (zfd.vitesse>STOP_SPEED-3) zfd.vitesse=STOP_SPEED-3 ;
			pzfd = &zfd ;
		}
	}
	
	// ... jusqu' extinction
	if (sintimeleft==0) {
		if (sinrate!=0) sinrate = sinrate * 4 / 5 ;
	}
	else
		sintimeleft -- ;

	// tout ceci ne sera fait qu'en cas de non-blocage
	if (lockvar==0)
	{
		// reperage de goom (acceleration forte de l'acceleration du volume)
		// -> coup de boost de la vitesse si besoin..
		if ( (accelvar>goomlimit) || (accelvar<-goomlimit) )
		{
			goomvar ++ ;
			if (goomvar%1 == 0)
			{
				int vtmp ;
				int newvit ;
				newvit = STOP_SPEED - speedvar / 2 ;

				// retablir le zoom avant..
				if ((zfd.reverse)&&(!(cycle%12))&&(rand ()%3==0)) {
					zfd.reverse = 0 ;
					zfd.vitesse = STOP_SPEED-2 ;
					lockvar = 50 ;
				}

				if (RAND () % 10 == 0)
				{
						zfd.reverse = 1;
						lockvar = 100;
				}
				
				// changement de milieu..
				switch (rand()%20)
				{
				case 0:
					zfd.middleY = HEIGHT - 1 ;
					zfd.middleX = WIDTH / 2 ;
					break ;
				case 1:
					zfd.middleX = WIDTH - 1 ;
					break ;
				case 2:
					zfd.middleX = 1 ;
					break ;
				default:
					zfd.middleY = HEIGHT / 2 ;
					zfd.middleX = WIDTH / 2 ;
				}

				switch (vtmp = (rand ()/100)%27)
				{
						case 0:
								zfd.vPlaneEffect = (rand () % 3) - (rand () % 3);
								zfd.hPlaneEffect = (rand () % 3) - (rand () % 3);
								break;
						case 3:
								zfd.vPlaneEffect = 0 ;
								zfd.hPlaneEffect = (rand () % 8) - (rand () % 8);
								break;
						case 4:
						case 5:
						case 6:
						case 7:
								zfd.vPlaneEffect = (rand()%5)-(rand()%5);
								zfd.hPlaneEffect = -zfd.vPlaneEffect;
								break;
						case 8:
								zfd.hPlaneEffect = 5 + (rand () % 8) ;
								zfd.vPlaneEffect = -zfd.hPlaneEffect ;
								break;
						case 9:
								zfd.vPlaneEffect = 5 + (rand () % 8) ;
								zfd.hPlaneEffect = -zfd.hPlaneEffect ;
								break;
						case 13:
								zfd.hPlaneEffect = 0;
								zfd.vPlaneEffect = (rand () % 10) - (rand () % 10);
								break;
						default:
								if (vtmp<10) {
									zfd.vPlaneEffect = 0 ;
									zfd.hPlaneEffect = 0 ;
								}
				}

				if (RAND () % 61 == 0)
				{
						if (rand () % 3) zfd.mode = NORMAL_MODE ;
						zfd.hPlaneEffect = RAND () % 32 + 32 ;
						lockvar = 100 ;
				}
				
				if (RAND () % 61 == 0)
				{
						if (rand () % 3) zfd.mode = NORMAL_MODE ;
						zfd.hPlaneEffect = -32 - (RAND () % 32) ;
						lockvar = 100 ;
				}

				/* noisifiage */
				if (RAND () % 56 == 0)
				{
						zfd.noisify = RAND () % 3 + 2 ;
						lockvar *= 3;
				}

				if (RAND () % 5 == 0)
						zfd.noisify = 0 ;

/*				if (rand () % 61 == 0)
				{
						zfd.mode = NORMAL_MODE ;
						zfd.hPlaneEffect = -256 ;
						zfd.vPlaneEffect = 256 ;
						zfd.vitesse = 109 ;
						zfd.reverse = 0 ;
						lockvar += 200 ;
				}*/
				
				if (zfd.mode == AMULETTE_MODE)
				{
						zfd.vPlaneEffect = 0;
						zfd.hPlaneEffect = 0;
				}

				if ((zfd.middleX == 1) || (zfd.middleX == WIDTH - 1))
				{
						zfd.vPlaneEffect = 0 ;
						zfd.hPlaneEffect = rand () % 2 ? 0 : zfd.hPlaneEffect;
				}

				if (newvit<zfd.vitesse) // on accelere
				{
					pzfd = &zfd ;
					if (((newvit<STOP_SPEED-7) && (zfd.vitesse<STOP_SPEED-6) && (cycle%3==0)) || (rand() % 40 == 0))
					{
						zfd.vitesse = STOP_SPEED-1 ;
						zfd.reverse = ! zfd.reverse ;
					}
					else {
						zfd.vitesse = (newvit + zfd.vitesse * 4) / 5 ;
					}
					printf ("speed -> %i --> new ZOOM : %i,%i\n", speedvar, zfd.vitesse, zfd.pertedec) ;
					lockvar += 50 ;
				}
			}
		}
		
		// mode mega-lent
		if (rand()%1000==0) {
			printf ("coup du sort...\n") ;
			pzfd = &zfd ;
			zfd.vitesse = STOP_SPEED-1 ;
			zfd.pertedec = 8 ;
			zfd.sqrtperte=16 ;
			goomvar = 1 ;
			lockvar += 70 ;
		}
	}
	
	// gros frein si la musique est calme
	if ((speedvar<1) && (zfd.vitesse<STOP_SPEED-4) && (cycle%16==0))
	{
		printf ("++slow part... %i\n", zfd.vitesse) ;
		pzfd = &zfd ;
		zfd.vitesse += 3 ;
		zfd.pertedec = 8 ;
		zfd.sqrtperte=16 ;
		goomvar = 0 ;
		printf ("--slow part... %i\n", zfd.vitesse) ;
	}
	
	// baisser regulierement la vitesse...
	if ( (cycle%73==0) && (zfd.vitesse<STOP_SPEED-5)) {
		printf ("slow down...\n") ;
		pzfd = &zfd ;
		zfd.vitesse ++ ;
	}
	
	// arreter de decrmenter au bout d'un certain temps
	if ((cycle%101==0) && (zfd.pertedec==7))
	{
		pzfd = &zfd ;
		zfd.pertedec=8 ;
		zfd.sqrtperte=16 ;
	}
	
	// passage du zoomFilter..
	if (pzfd) {
		printf (" ** %i - %i,%i - %i - %i **\n",
			pzfd->mode,
			pzfd->middleX, pzfd->middleY, pzfd->vitesse,
			pzfd->sqrtperte) ;
	}
	zoomFilterFastRGB (p1, p2, pzfd) ;

	// si on est dans un goom : afficher l'oscillo...
	if (agoom>0)
	{
		Uint *p ;
		Uint vcolor, rcolor ;
		if (agoom>1) {
			p=p2;
			switch (agoom) {
			case 4 :
				vcolor = 0x0014ff82 ;
				rcolor = 0x00ff4228 ;
				break ;
			case 3 :
				vcolor = 0x0014cc82 ;
				rcolor = 0x00cc4228 ;
				break ;
			case 2 :
				vcolor = 0x00ddffdd ;
				rcolor = 0x00ffdddd ;
				break ;
			default :
				vcolor = 0x0066ee66 ;
				rcolor = 0x00ee6666 ;
			}
		}
		else {
			p=p1 ;
			vcolor = 0x0014aa82;
			rcolor = 0x00aa4228;
		}

		for (i=0;i<512;i++)
		{
			unsigned int plot ;
			#ifndef USE_ASM
				plot=i*WIDTH/512 + (HEIGHT/4+data[0][i]/2000)*WIDTH ;
				if (plot<SIZE)
					p[plot] = vcolor;
				plot=i*WIDTH/512 + (HEIGHT*3/4-data[1][i]/2000)*WIDTH ;
				if (plot<SIZE)
					p[plot] = rcolor;
			#else
				plot=i*WIDTH/512 + (HEIGHT/4+data[0][i]/1600)*WIDTH ;
				if (plot<SIZE-1) {
					p[plot] = vcolor;
					p[plot+1] = vcolor;
				}
				plot=i*WIDTH/512 + (HEIGHT*3/4-data[1][i]/1600)*WIDTH ;
				if (plot<SIZE-1) {
					p[plot] = rcolor;
					p[plot+1] = rcolor;
				}
			#endif
		}
	}

	// si l'intensit du filter sin est > 0 : faire un passage...
#ifndef USE_ASM
	if (sinrate>0) {
		sinFilter(p2, p1, cycle, SIN_ADD, sinrate, 5 , 17);
		ptc_update (p1) ;
		tmp=p1;
		p1=p2;
		p2=tmp;
	}
#else
	if (sinrate>0) {
		sinFilter(p2, p1, cycle, SIN_ADD, sinrate , 5 , 17);
		sinrate=0;
		ptc_update (p1) ;
	}
#endif
	else {
		ptc_update (p2) ;
		tmp=p1;
		p1=p2;
		p2=tmp;
	}

	// affichage et swappage des buffers..
	cycle++;
	
	// tous les 100 cycles : vrifier si le taux de goom est correct
	// et le modifier sinon..
	if (!(cycle%100))
	{
		printf ("gooms : %i -- grosgoom : %i\n",totalgoom, totalgrosgoom);
		if (totalgoom>15)
		{
			printf ("less gooms\n") ;
			goomlimit ++ ;
		}
		else
		{
			if ((totalgoom==0) && (goomlimit>1))
			{
				printf ("more gooms\n") ;
				goomlimit -- ;
			}
		
			if (totalgrosgoom>2)
			{
				printf ("less grosgooms\n") ;
				grosgoomlimit ++ ;
			}
		}
		if ((totalgrosgoom==0) && (goomlimit>2))
		{
			printf ("more grosgooms\n") ;
			grosgoomlimit -- ;
		}
		
		totalgoom = 0 ;
		totalgrosgoom = 0 ;
	}
}

void jeko_update_config ()
{
	WIDTH = extern_config.xres ;
	HEIGHT = extern_config.yres ;
	SIZE = HEIGHT * WIDTH ;

	POINT_WIDTH = 2 * WIDTH / 5 ;
	POINT_HEIGHT = 2 * HEIGHT / 5 ;
}
